@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python
 # SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under under a slightly modified version
@@ -17,7 +17,8 @@
 #
 #     U2U: https://tools.ietf.org/html/draft-ietf-cat-user2user-02
 #     [MS-SFU]: https://msdn.microsoft.com/en-us/library/cc246071.aspx
-
+from __future__ import division
+from __future__ import print_function
 import argparse
 import datetime
 import logging
@@ -25,6 +26,8 @@ import random
 import re
 import struct
 import sys
+from binascii import unhexlify
+from six import b
 
 from pyasn1.codec.der import decoder, encoder
 from pyasn1.type.univ import noValue
@@ -50,14 +53,14 @@ class S4U2SELF:
         adIfRelevant = decoder.decode(encTicketPart['authorization-data'][0]['ad-data'], asn1Spec=AD_IF_RELEVANT())[
             0]
         # So here we have the PAC
-        pacType = PACTYPE(str(adIfRelevant[0]['ad-data']))
+        pacType = PACTYPE(adIfRelevant[0]['ad-data'].asOctets())
         buff = pacType['Buffers']
 
         for bufferN in range(pacType['cBuffers']):
             infoBuffer = PAC_INFO_BUFFER(buff)
             data = pacType['Buffers'][infoBuffer['Offset']-8:][:infoBuffer['cbBufferSize']]
             if logging.getLogger().level == logging.DEBUG:
-                print "TYPE 0x%x" % infoBuffer['ulType']
+                print("TYPE 0x%x" % infoBuffer['ulType'])
             if infoBuffer['ulType'] == 1:
                 type1 = TypeSerialization1(data)
                 # I'm skipping here 4 bytes with its the ReferentID for the pointer
@@ -66,35 +69,35 @@ class S4U2SELF:
                 kerbdata.fromString(newdata)
                 kerbdata.fromStringReferents(newdata[len(kerbdata.getData()):])
                 kerbdata.dump()
-                print
-                print 'Domain SID:', kerbdata['LogonDomainId'].formatCanonical()
-                print
+                print()
+                print('Domain SID:', kerbdata['LogonDomainId'].formatCanonical())
+                print()
             elif infoBuffer['ulType'] == PAC_CLIENT_INFO_TYPE:
                 clientInfo = PAC_CLIENT_INFO(data)
                 if logging.getLogger().level == logging.DEBUG:
                     clientInfo.dump()
-                    print
+                    print()
             elif infoBuffer['ulType'] == PAC_SERVER_CHECKSUM:
                 signatureData = PAC_SIGNATURE_DATA(data)
                 if logging.getLogger().level == logging.DEBUG:
                     signatureData.dump()
-                    print
+                    print()
             elif infoBuffer['ulType'] == PAC_PRIVSVR_CHECKSUM:
                 signatureData = PAC_SIGNATURE_DATA(data)
                 if logging.getLogger().level == logging.DEBUG:
                     signatureData.dump()
-                    print
+                    print()
             elif infoBuffer['ulType'] == PAC_UPN_DNS_INFO:
                 upn = UPN_DNS_INFO(data)
                 if logging.getLogger().level == logging.DEBUG:
                     upn.dump()
-                    print data[upn['DnsDomainNameOffset']:]
-                    print
+                    print(data[upn['DnsDomainNameOffset']:])
+                    print()
             else:
                 hexdump(data)
 
             if logging.getLogger().level == logging.DEBUG:
-                print "#"*80
+                print("#"*80)
 
             buff = buff[len(infoBuffer):]
 
@@ -114,7 +117,7 @@ class S4U2SELF:
 
         userName = Principal(self.__username, type=constants.PrincipalNameType.NT_PRINCIPAL.value)
         tgt, cipher, oldSessionKey, sessionKey = getKerberosTGT(userName, self.__password, self.__domain,
-                                                                self.__lmhash.decode('hex'), self.__nthash.decode('hex'))
+                                                                unhexlify(self.__lmhash), unhexlify(self.__nthash))
 
         decodedTGT = decoder.decode(tgt, asn1Spec = AS_REP())[0]
 
@@ -145,7 +148,7 @@ class S4U2SELF:
 
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('AUTHENTICATOR')
-            print authenticator.prettyPrint()
+            print(authenticator.prettyPrint())
             print ('\n')
 
         encodedAuthenticator = encoder.encode(authenticator)
@@ -178,7 +181,7 @@ class S4U2SELF:
         clientName = Principal(self.__behalfUser, type=constants.PrincipalNameType.NT_PRINCIPAL.value)
 
         S4UByteArray = struct.pack('<I',constants.PrincipalNameType.NT_PRINCIPAL.value)
-        S4UByteArray += self.__behalfUser + self.__domain + 'Kerberos'
+        S4UByteArray += b(self.__behalfUser) + b(self.__domain) + b'Kerberos'
 
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('S4UByteArray')
@@ -204,7 +207,7 @@ class S4U2SELF:
 
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('PA_FOR_USER_ENC')
-            print paForUserEnc.prettyPrint()
+            print(paForUserEnc.prettyPrint())
 
         encodedPaForUserEnc = encoder.encode(paForUserEnc)
 
@@ -242,7 +245,7 @@ class S4U2SELF:
 
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('Final TGS')
-            print tgsReq.prettyPrint()
+            print(tgsReq.prettyPrint())
 
         message = encoder.encode(tgsReq)
 
@@ -252,7 +255,7 @@ class S4U2SELF:
 
         if logging.getLogger().level == logging.DEBUG:
             logging.debug('TGS_REP')
-            print tgs.prettyPrint()
+            print(tgs.prettyPrint())
 
         cipherText = tgs['ticket']['enc-part']['cipher']
 
@@ -264,8 +267,8 @@ class S4U2SELF:
         newCipher = _enctype_table[int(tgs['ticket']['enc-part']['etype'])]
 
         # Pass the hash/aes key :P
-        if self.__nthash != '':
-            key = Key(newCipher.enctype, self.__nthash.decode('hex'))
+        if self.__nthash != '' and (isinstance(self.__nthash, bytes) and self.__nthash != b''):
+            key = Key(newCipher.enctype, unhexlify(self.__nthash))
         else:
             if newCipher.enctype == Enctype.RC4:
                 key = newCipher.string_to_key(password, '', None)
@@ -277,14 +280,14 @@ class S4U2SELF:
             plainText = newCipher.decrypt(key, 2, str(cipherText))
         except:
             # S4USelf + U2U uses this other key
-            plainText = cipher.decrypt(sessionKey, 2, str(cipherText))
+            plainText = cipher.decrypt(sessionKey, 2, cipherText)
 
         self.printPac(plainText)
 
 # Process command-line arguments.
 if __name__ == '__main__':
     logger.init()
-    print version.BANNER
+    print(version.BANNER)
 
     parser = argparse.ArgumentParser()
 
@@ -322,15 +325,16 @@ if __name__ == '__main__':
 
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
+        # Print the Library's installation path
+        logging.debug(version.getInstallationPath())
     else:
         logging.getLogger().setLevel(logging.INFO)
 
     try:
         dumper = S4U2SELF(options.targetUser, username, password, domain, options.hashes)
         dumper.dump(address)
-    except Exception, e:
+    except Exception as e:
         if logging.getLogger().level == logging.DEBUG:
             import traceback
             traceback.print_exc()
         logging.error(str(e))
-
