@@ -936,7 +936,7 @@ class MS14_068:
                         # Windows XP). So, if that's the case we'll force using RC4 by converting
                         # the password to lm/nt hashes and hope for the best. If that's already
                         # done, byebye.
-                        if self.__lmhash is '' and self.__nthash is '':
+                        if self.__lmhash == '' and self.__nthash == '':
                             from impacket.ntlm import compute_lmhash, compute_nthash
                             self.__lmhash = compute_lmhash(self.__password)
                             self.__nthash = compute_nthash(self.__password)
@@ -991,7 +991,7 @@ class MS14_068:
                         # Windows XP). So, if that's the case we'll force using RC4 by converting
                         # the password to lm/nt hashes and hope for the best. If that's already
                         # done, byebye.
-                        if self.__lmhash is '' and self.__nthash is '':
+                        if self.__lmhash == '' and self.__nthash == '':
                             from impacket.ntlm import compute_lmhash, compute_nthash
                             self.__lmhash = compute_lmhash(self.__password)
                             self.__nthash = compute_nthash(self.__password)
@@ -1036,8 +1036,6 @@ class MS14_068:
                 executer.run(self.__target)
 
 if __name__ == '__main__':
-    # Init the example's logger theme
-    logger.init()
     import argparse
     import sys
     try:
@@ -1073,6 +1071,7 @@ if __name__ == '__main__':
                                                  "target or saves the TGT for later use.")
 
     parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName>')
+    parser.add_argument('-ts', action='store_true', help='Adds timestamp to every logging output')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('command', nargs='*', default=' ',
                         help='command (or arguments if -c is used) to execute at the target (w/o path). Defaults to '
@@ -1106,6 +1105,9 @@ if __name__ == '__main__':
  
     options = parser.parse_args()
 
+    # Init the example's logger theme
+    logger.init(options.ts)
+
     import re
 
     domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
@@ -1116,12 +1118,14 @@ if __name__ == '__main__':
         password = password + '@' + address.rpartition('@')[0]
         address = address.rpartition('@')[2]
 
-    if domain is '':
+    if domain == '':
         logging.critical('Domain should be specified!')
         sys.exit(1)
 
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
+        # Print the Library's installation path
+        logging.debug(version.getInstallationPath())
     else:
         logging.getLogger().setLevel(logging.INFO)
 
