@@ -1,4 +1,4 @@
-#!/usr/bin/python2
+#!/usr/bin/python
 """ifmap - scan for listening DCERPC interfaces
 
 Usage: ifmap.py hostname port
@@ -33,6 +33,8 @@ listening on any port. This is imposed by the DCERPC spec.
 
 Author: Catalin Patulea <cat@vv.carleton.ca>
 """
+from __future__ import division
+from __future__ import print_function
 import sys
 import struct
 
@@ -282,21 +284,21 @@ ffe561b8-bf15-11cf-8c5e-08002bb49649 v2.0
 uuid_database = set((uuidstr.upper(), ver) for uuidstr, ver in uuid_database)
 
 # add the ones from ndrutils
-k = KNOWN_UUIDS.keys()[0]
+k = list(KNOWN_UUIDS.keys())[0]
 def fix_ndr_uuid(ndruuid):
   assert len(ndruuid) == 18
   uuid = ndruuid[:16]
   maj, min = struct.unpack("BB", ndruuid[16:])
   return uuid + struct.pack("<HH", maj, min)
 uuid_database.update(
-  uuid.bin_to_uuidtup(fix_ndr_uuid(bin)) for bin in KNOWN_UUIDS.keys()
+  uuid.bin_to_uuidtup(fix_ndr_uuid(bin)) for bin in list(KNOWN_UUIDS.keys())
 )
 
 def main(args):
   # Init the example's logger theme
   logger.init()
   if len(args) != 2:
-    print "usage: ./ifmap.py <host> <port>"
+    print("usage: ./ifmap.py <host> <port>")
     return 1
 
   host = args[0]
@@ -329,7 +331,7 @@ def main(args):
     binuuid = uuid.uuidtup_to_bin(tup)
     try:
       dce.bind(binuuid)
-    except rpcrt.DCERPCException, e:
+    except rpcrt.DCERPCException as e:
       if str(e).find('abstract_syntax_not_supported') >= 0:
         listening = False
       else:
@@ -340,20 +342,20 @@ def main(args):
     listed = tup in uuidtups
     otherversion = any(tup[0] == uuidstr for uuidstr, ver in uuidtups)
     if listed or listening:
-      if epm.KNOWN_PROTOCOLS.has_key(tup[0]):
-          print "Protocol: %s" % (epm.KNOWN_PROTOCOLS[tup[0]])
+      if tup[0] in epm.KNOWN_PROTOCOLS:
+          print("Protocol: %s" % (epm.KNOWN_PROTOCOLS[tup[0]]))
       else:
-          print "Procotol: N/A"
+          print("Procotol: N/A")
 
-      if KNOWN_UUIDS.has_key(uuid.uuidtup_to_bin(tup)[:18]):
-          print "Provider: %s" % (KNOWN_UUIDS[uuid.uuidtup_to_bin(tup)[:18]])
+      if uuid.uuidtup_to_bin(tup)[:18] in KNOWN_UUIDS:
+          print("Provider: %s" % (KNOWN_UUIDS[uuid.uuidtup_to_bin(tup)[:18]]))
       else:
-          print "Provider: N/A"
-      print "UUID     : %s v%s: %s, %s\n" % (
+          print("Provider: N/A")
+      print("UUID     : %s v%s: %s, %s\n" % (
         tup[0], tup[1],
         "listed" if listed else "other version listed" if otherversion else "not listed",
         "listening" if listening else "not listening"
-      )
+      ))
 
 
 if __name__ == "__main__":
