@@ -19,6 +19,7 @@
 #   There are test cases for them too. 
 #
 from struct import pack
+from six import b
 from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDRENUM, NDRUNION, NDRPOINTER, NDRUniConformantArray, \
     NDRUniFixedArray, NDRUniConformantVaryingArray
 from impacket.dcerpc.v5.dtypes import WSTR, LPWSTR, DWORD, ULONG, USHORT, PGUID, NTSTATUS, NULL, LONG, UCHAR, PRPC_SID, \
@@ -31,10 +32,11 @@ from impacket.dcerpc.v5.lsad import PLSA_FOREST_TRUST_INFORMATION
 from impacket.dcerpc.v5.rpcrt import DCERPCException
 from impacket.structure import Structure
 from impacket import ntlm, crypto, LOG
-import hmac, hashlib
+import hmac
+import hashlib
 try:
     from Cryptodome.Cipher import DES, AES, ARC4
-except Exception:
+except ImportError:
     LOG.critical("Warning: You don't have any crypto installed. You need pycryptodomex")
     LOG.critical("See https://pypi.org/project/pycryptodomex/")
 
@@ -46,11 +48,11 @@ class DCERPCSessionError(DCERPCException):
 
     def __str__( self ):
         key = self.error_code
-        if system_errors.ERROR_MESSAGES.has_key(key):
+        if key in system_errors.ERROR_MESSAGES:
             error_msg_short = system_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = system_errors.ERROR_MESSAGES[key][1] 
             return 'NRPC SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
-        elif nt_errors.ERROR_MESSAGES.has_key(key):
+        elif key in nt_errors.ERROR_MESSAGES:
             error_msg_short = nt_errors.ERROR_MESSAGES[key][0]
             error_msg_verbose = nt_errors.ERROR_MESSAGES[key][1] 
             return 'NRPC SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
@@ -136,7 +138,7 @@ PLOGONSRV_HANDLE = LPWSTR
 # 2.2.1.1.1 CYPHER_BLOCK
 class CYPHER_BLOCK(NDRSTRUCT):
     structure = (
-        ('Data', '8s=""'),
+        ('Data', '8s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -1578,7 +1580,7 @@ class NL_AUTH_MESSAGE(Structure):
     def __init__(self, data = None, alignment = 0):
         Structure.__init__(self, data, alignment)
         if data is None:
-            self['Buffer'] = '\x00'*4
+            self['Buffer'] = b'\x00'*4
 
 class NL_AUTH_SIGNATURE(Structure):
     structure = (
@@ -1654,11 +1656,11 @@ def ComputeSessionKeyStrongKey(sharedSecret, clientChallenge, serverChallenge, s
         M4SS = sharedSecretHash
 
     md5 = hashlib.new('md5')
-    md5.update('\x00'*4)
+    md5.update(b'\x00'*4)
     md5.update(clientChallenge)
     md5.update(serverChallenge)
     finalMD5 = md5.digest()
-    hm = hmac.new(M4SS) 
+    hm = hmac.new(M4SS, digestmod=hashlib.md5)
     hm.update(finalMD5)
     return hm.digest()
 
@@ -1674,31 +1676,31 @@ def deriveSequenceNumber(sequenceNum):
 def ComputeNetlogonSignatureAES(authSignature, message, confounder, sessionKey):
     # [MS-NRPC] Section 3.3.4.2.1, point 7
     hm = hmac.new(key=sessionKey, digestmod=hashlib.sha256)
-    hm.update(str(authSignature)[:8])
+    hm.update(authSignature.getData()[:8])
     # If no confidentiality requested, it should be ''
     hm.update(confounder)
-    hm.update(str(message))
+    hm.update(bytes(message))
     return hm.digest()[:8]+'\x00'*24
 
 def ComputeNetlogonSignatureMD5(authSignature, message, confounder, sessionKey):
     # [MS-NRPC] Section 3.3.4.2.1, point 7
     md5 = hashlib.new('md5')
-    md5.update('\x00'*4)
-    md5.update(str(authSignature)[:8])
+    md5.update(b'\x00'*4)
+    md5.update(authSignature.getData()[:8])
     # If no confidentiality requested, it should be ''
     md5.update(confounder)
-    md5.update(str(message))
+    md5.update(bytes(message))
     finalMD5 = md5.digest()
-    hm = hmac.new(sessionKey)
+    hm = hmac.new(sessionKey, digestmod=hashlib.md5)
     hm.update(finalMD5)
     return hm.digest()[:8]
 
 def encryptSequenceNumberRC4(sequenceNum, checkSum, sessionKey):
     # [MS-NRPC] Section 3.3.4.2.1, point 9
 
-    hm = hmac.new(sessionKey)
-    hm.update('\x00'*4)
-    hm2 = hmac.new(hm.digest())
+    hm = hmac.new(sessionKey, digestmod=hashlib.md5)
+    hm.update(b'\x00'*4)
+    hm2 = hmac.new(hm.digest(), digestmod=hashlib.md5)
     hm2.update(checkSum)
     encryptionKey = hm2.digest()
 
@@ -1747,15 +1749,17 @@ def SIGN(data, confounder, sequenceNum, key, aes = False):
 def SEAL(data, confounder, sequenceNum, key, aes = False):
     signature = SIGN(data, confounder, sequenceNum, key, aes)
     sequenceNum = deriveSequenceNumber(sequenceNum)
-    XorKey = []
-    for i in key:
-       XorKey.append(chr(ord(i) ^ 0xf0))
 
-    XorKey = ''.join(XorKey)
+    XorKey = bytearray(key)
+    for i in range(len(XorKey)):
+        XorKey[i] = XorKey[i] ^ 0xf0
+
+    XorKey = bytes(XorKey)
+
     if aes is False:
-        hm = hmac.new(XorKey)
-        hm.update('\x00'*4)
-        hm2 = hmac.new(hm.digest())
+        hm = hmac.new(XorKey, digestmod=hashlib.md5)
+        hm.update(b'\x00'*4)
+        hm2 = hmac.new(hm.digest(), digestmod=hashlib.md5)
         hm2.update(sequenceNum)
         encryptionKey = hm2.digest()
 
@@ -1779,16 +1783,17 @@ def SEAL(data, confounder, sequenceNum, key, aes = False):
         
 def UNSEAL(data, auth_data, key, aes = False):
     auth_data = NL_AUTH_SIGNATURE(auth_data)
-    XorKey = []
-    for i in key:
-       XorKey.append(chr(ord(i) ^ 0xf0))
+    XorKey = bytearray(key)
+    for i in range(len(XorKey)):
+        XorKey[i] = XorKey[i] ^ 0xf0
+
+    XorKey = bytes(XorKey)
 
-    XorKey = ''.join(XorKey)
     if aes is False:
         sequenceNum = decryptSequenceNumberRC4(auth_data['SequenceNumber'], auth_data['Checksum'],  key)
-        hm = hmac.new(XorKey)
-        hm.update('\x00'*4)
-        hm2 = hmac.new(hm.digest())
+        hm = hmac.new(XorKey, digestmod=hashlib.md5)
+        hm.update(b'\x00'*4)
+        hm2 = hmac.new(hm.digest(), digestmod=hashlib.md5)
         hm2.update(sequenceNum)
         encryptionKey = hm2.digest()
 
@@ -1810,24 +1815,26 @@ def UNSEAL(data, auth_data, key, aes = False):
 def getSSPType1(workstation='', domain='', signingRequired=False):
     auth = NL_AUTH_MESSAGE()
     auth['Flags'] = 0
-    auth['Buffer'] = ''
+    auth['Buffer'] = b''
     auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_DOMAIN 
     if domain != '':
-        auth['Buffer'] = auth['Buffer'] + domain + '\x00'
+        auth['Buffer'] = auth['Buffer'] + b(domain) + b'\x00'
     else:
-        auth['Buffer'] += 'WORKGROUP\x00'
+        auth['Buffer'] += b'WORKGROUP\x00'
 
-    auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_HOST 
-    if workstation != '':
-        auth['Buffer'] = auth['Buffer'] + workstation + '\x00'
-    else:
-        auth['Buffer'] += 'MYHOST\x00'
+    auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_HOST
 
-    auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_HOST_UTF8 
     if workstation != '':
-        auth['Buffer'] += pack('<B',len(workstation)) + workstation + '\x00'
+        auth['Buffer'] = auth['Buffer'] + b(workstation) + b'\x00'
     else:
-        auth['Buffer'] += '\x06MYHOST\x00'
+        auth['Buffer'] += b'MYHOST\x00'
+
+    auth['Flags'] |= NL_AUTH_MESSAGE_NETBIOS_HOST_UTF8
+
+    if workstation != '':
+        auth['Buffer'] += pack('<B',len(workstation)) + b(workstation) + b'\x00'
+    else:
+        auth['Buffer'] += b'\x06MYHOST\x00'
 
     return auth
 
@@ -2764,7 +2771,7 @@ def hNetrLogonGetDomainInfo(dce, serverName, computerName, authenticator, return
     request['ComputerName'] = checkNullString(computerName)
     request['Authenticator'] = authenticator
     if returnAuthenticator == 0:
-        request['ReturnAuthenticator']['Credential'] = '\x00'*8
+        request['ReturnAuthenticator']['Credential'] = b'\x00'*8
         request['ReturnAuthenticator']['Timestamp'] = 0
     else:
         request['ReturnAuthenticator'] = returnAuthenticator
@@ -2790,7 +2797,7 @@ def hNetrLogonGetCapabilities(dce, serverName, computerName, authenticator, retu
     request['ComputerName'] = checkNullString(computerName)
     request['Authenticator'] = authenticator
     if returnAuthenticator == 0:
-        request['ReturnAuthenticator']['Credential'] = '\x00'*8
+        request['ReturnAuthenticator']['Credential'] = b'\x00'*8
         request['ReturnAuthenticator']['Timestamp'] = 0
     else:
         request['ReturnAuthenticator'] = returnAuthenticator
@@ -2805,6 +2812,3 @@ def hNetrServerGetTrustInfo(dce, trustedDcName, accountName, secureChannelType,
     request['ComputerName'] = checkNullString(computerName)
     request['Authenticator'] = authenticator
     return dce.request(request)
-
-
-
