@@ -18,15 +18,10 @@
 #   Helper functions start with "h"<name of the call>.
 #   There are test cases for them too. 
 #
-
-
-from struct import unpack, pack
-
-from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantVaryingArray, NDRUniConformantArray
-from impacket.dcerpc.v5.dtypes import ULONGLONG, UINT, USHORT, LPWSTR, DWORD, UUID, ULONG, LPULONG, BOOLEAN, SECURITY_INFORMATION, PFILETIME, \
-    RPC_UNICODE_STRING, FILETIME, NULL, MAXIMUM_ALLOWED, OWNER_SECURITY_INFORMATION, PWCHAR, PRPC_UNICODE_STRING
+from impacket import system_errors
+from impacket.dcerpc.v5.dtypes import ULONGLONG, UINT, USHORT, LPWSTR, DWORD, ULONG, NULL
+from impacket.dcerpc.v5.ndr import NDRCALL, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantArray
 from impacket.dcerpc.v5.rpcrt import DCERPCException
-from impacket import system_errors, LOG
 from impacket.uuid import uuidtup_to_bin
 
 MSRPC_UUID_RPRN = uuidtup_to_bin(('12345678-1234-ABCD-EF00-0123456789AB', '1.0'))
@@ -500,7 +495,7 @@ def hRpcEnumPrinters(dce, flags, name = NULL, level = 1):
     Full Documentation: https://msdn.microsoft.com/en-us/library/cc244794.aspx
 
     :param DCERPC_v5 dce: a connected DCE instance.
-    :param DWORD flags: The types of print objects that this method enumerates. The value of this parameter is the
+    :param int flags: The types of print objects that this method enumerates. The value of this parameter is the
     result of a bitwise OR of one or more of the Printer Enumeration Flags (section 2.2.3.7).
     :param string name: NULL or a server name parameter as specified in Printer Server Name Parameters (section 3.1.4.1.4).
     :param level: The level of printer information structure.
@@ -514,7 +509,7 @@ def hRpcEnumPrinters(dce, flags, name = NULL, level = 1):
     request['Level'] = level
     bytesNeeded = 0
     try:
-        resp = dce.request(request)
+        dce.request(request)
     except DCERPCSessionError as e:
         if str(e).find('ERROR_INSUFFICIENT_BUFFER') < 0:
             raise
@@ -528,4 +523,3 @@ def hRpcEnumPrinters(dce, flags, name = NULL, level = 1):
     request['cbBuf'] = bytesNeeded
     request['pPrinterEnum'] = b'a' * bytesNeeded
     return dce.request(request)
-
